@@ -26,8 +26,9 @@ except ImportError:
     localz = None
 
 log = logging.getLogger(__name__)
-Latest = None  # Enum
-NoVersion = None
+
+Latest = model.Latest  # Enum
+NoVersion = model.NoVersion
 
 
 class State(dict):
@@ -47,7 +48,7 @@ class State(dict):
             "profileName": storage.value("startupProfile"),
             "appRequest": storage.value("startupApplication"),
 
-            # String or callable, returning list of profile names
+            # list or callable, returning list of profile names
             "root": None,
 
             # Current error, if any
@@ -208,11 +209,11 @@ class Controller(QtCore.QObject):
         state = State(self, storage)
 
         models = {
-            "profileVersions": QtCore.QStringListModel(),
-            "profileNames": QtCore.QStringListModel(),
             "apps": model.ApplicationModel(),
+            "profileVersions": QtCore.QStringListModel(),
 
             # Docks
+            "profiles": model.ProfileModel(),
             "packages": model.PackagesModel(self),
             "context": model.ContextModel(),
             "environment": model.EnvironmentModel(),
@@ -385,7 +386,7 @@ class Controller(QtCore.QObject):
             # package family not found: occoc (searched: C:\)
             _, package, paths = value.value.split(": ", 2)
             package = package.split(" (", 1)[0]
-            paths = paths.rstrip(")").split(os.pathsep)
+            paths = paths.rstrip(")").split("; ")  # Hard-coded pathsep in Rez
 
             message = """
                 <h2><font color=\"red\">:(</font></h2>
@@ -444,7 +445,7 @@ class Controller(QtCore.QObject):
 
         Arguments:
             family (str): Name of package
-            range (str): Range, e.g. "1" or "==0.3.13"
+            range_ (str): Range, e.g. "1" or "==0.3.13"
 
         """
 
@@ -552,8 +553,9 @@ class Controller(QtCore.QObject):
         with its corresponding Rez package.
 
         Arguments:
-            root (str): Absolute path to profiles on disk, or callable
-                returning names of profiles
+            root (list, callable): A list of profile names, or a callable
+                returning names of profiles.
+            on_success (callable): Callback on reset completed.
 
         """
 
@@ -588,8 +590,6 @@ class Controller(QtCore.QObject):
                 default_profile = name
 
             self._state["rezProfiles"].update(profiles)
-            self._models["profileNames"].setStringList(list(profiles))
-            self._models["profileNames"].layoutChanged.emit()
 
             # On resetting after startup, there will be a
             # currently selected profile that may differ from
@@ -606,6 +606,10 @@ class Controller(QtCore.QObject):
             if not current_profile:
                 current_profile = default_profile
 
+            self._models["profiles"].set_favorites(self)
+            self._models["profiles"].set_current(current_profile)
+            self._models["profiles"].reset(profiles)
+
             self._state["profileName"] = current_profile
             self._state["root"] = root
 
@@ -811,6 +815,10 @@ class Controller(QtCore.QObject):
                 self.error("Could not find profiles in %s" % root)
                 profiles = []
 
+        else:
+            raise TypeError("Argument 'root' should be either list type or "
+                            "callable.")
+
         # Facilitate accidental empty family names, e.g. None or ''
         profiles = list(filter(None, profiles))
 
@@ -941,11 +949,11 @@ class Controller(QtCore.QObject):
     def _package_paths(self):
         """Return all package paths, relative the current state of the world"""
 
-        paths = util.normpaths(*rez.config.packages_path)
+        paths = rez.config.packages_path[:]
 
         # Optional development packages
         if not self._state.retrieve("useDevelopmentPackages"):
-            paths = util.normpaths(*rez.config.nonlocal_packages_path)
+            paths = rez.config.nonlocal_packages_path[:]
 
         # Optional package localisation
         if localz and not self._state.retrieve("useLocalizedPackages", True):
@@ -998,7 +1006,7 @@ class Controller(QtCore.QObject):
         contexts = odict()
         with util.timing() as t:
             for app_request in apps:
-                app_request = rez.PackageRequest(app_request)
+                app_request = rez.PackageRequest(app_request.strip("~"))
                 app_package = rez.find_latest(app_request.name,
                                               range_=app_request.range)
 
