@@ -1,6 +1,7 @@
 import os
 import re
 import time
+import json
 import logging
 import collections
 from itertools import chain
@@ -67,7 +68,7 @@ class AbstractDockWidget(QtWidgets.QDockWidget):
 
 
 class App(AbstractDockWidget):
-    """Aggregated information about the currently selected profile"""
+    """Aggregated information about the currently selected application"""
 
     icon = "Alert_Info_32"
 
@@ -244,6 +245,8 @@ class Console(AbstractDockWidget):
             logging.CRITICAL: "<font color=\"red\">",
         }.get(level, "<font color=\"#222\">")
 
+        line = line.replace(" ", "&nbsp;")
+        line = line.replace("\n", "<br>")
         line = "%s%s</font><br>" % (color, line)
 
         cursor = self._widgets["text"].textCursor()
@@ -462,13 +465,15 @@ class Packages(AbstractDockWidget):
 
         def on_openfile():
             package = model_.data(index, "package")
-            fname = os.path.join(package.root, "package.py")
+            pkg_uri = os.path.dirname(package.uri)
+            fname = os.path.join(pkg_uri, "package.py")
             util.open_file_location(fname)
             self.message.emit("Opened %s" % fname)
 
         def on_copyfile():
             package = model_.data(index, "package")
-            fname = os.path.join(package.root, "package.py")
+            pkg_uri = os.path.dirname(package.uri)
+            fname = os.path.join(pkg_uri, "package.py")
             clipboard = QtWidgets.QApplication.instance().clipboard()
             clipboard.setText(fname)
             self.message.emit("Copied %s" % fname)
@@ -525,7 +530,7 @@ class Context(AbstractDockWidget):
         }
 
         widgets = {
-            "view": QtWidgets.QTreeView(),
+            "view": JsonView(),
             "graph": QtImageViewer.QtImageViewer(),
             "generateGraph": QtWidgets.QPushButton("Update"),
             "graphHotkeys": QtWidgets.QLabel(),
@@ -627,19 +632,19 @@ class Environment(AbstractDockWidget):
         }
 
         pages = {
-            "environment": QtWidgets.QTreeView(),
+            "environment": QtWidgets.QWidget(),
             "editor": EnvironmentEditor(),
         }
 
         widgets = {
+            "view": JsonView(),
         }
 
-        layout = QtWidgets.QVBoxLayout(panels["central"])
+        layout = QtWidgets.QVBoxLayout(pages["environment"])
         layout.setContentsMargins(0, 0, 0, 0)
-        layout.addWidget(pages["environment"])
+        layout.addWidget(widgets["view"])
 
-        pages["environment"].setSortingEnabled(True)
-        self.setWidget(panels["central"])
+        widgets["view"].setSortingEnabled(True)
 
         pages["editor"].applied.connect(self.on_env_applied)
 
@@ -650,6 +655,8 @@ class Environment(AbstractDockWidget):
         pages["editor"].from_environment(user_env)
         pages["editor"].warning.connect(self.on_env_warning)
 
+        self.setWidget(panels["central"])
+
         self._ctrl = ctrl
         self._panels = panels
         self._pages = pages
@@ -658,7 +665,7 @@ class Environment(AbstractDockWidget):
     def set_model(self, model_):
         proxy_model = QtCore.QSortFilterProxyModel()
         proxy_model.setSourceModel(model_)
-        self._pages["environment"].setModel(proxy_model)
+        self._widgets["view"].setModel(proxy_model)
 
     def on_env_applied(self, env):
         self._ctrl.state.store("userEnv", env)
@@ -668,6 +675,54 @@ class Environment(AbstractDockWidget):
         self._ctrl.warning(message)
 
 
+class JsonView(QtWidgets.QTreeView):
+    def __init__(self, parent=None):
+        super(JsonView, self).__init__(parent)
+        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self.customContextMenuRequested.connect(self.on_right_click)
+
+    def on_right_click(self, position):
+        index = self.indexAt(position)
+
+        if not index.isValid():
+            # Clicked outside any item
+            return
+
+        model_ = index.model()
+        menu = MenuWithTooltip(self)
+        copy = QtWidgets.QAction("Copy JSON", menu)
+        copy_full = QtWidgets.QAction("Copy full JSON", menu)
+
+        menu.addAction(copy)
+        menu.addAction(copy_full)
+        menu.addSeparator()
+
+        def on_copy():
+            text = str(model_.data(index, model.JsonModel.JsonRole))
+            app = QtWidgets.QApplication.instance()
+            app.clipboard().setText(text)
+
+        def on_copy_full():
+            if isinstance(model_, QtCore.QSortFilterProxyModel):
+                data = model_.sourceModel().json()
+            else:
+                data = model_.json()
+
+            text = json.dumps(data,
+                              indent=4,
+                              sort_keys=True,
+                              ensure_ascii=False)
+
+            app = QtWidgets.QApplication.instance()
+            app.clipboard().setText(text)
+
+        copy.triggered.connect(on_copy)
+        copy_full.triggered.connect(on_copy_full)
+
+        menu.move(QtGui.QCursor.pos())
+        menu.show()
+
+
 class TextEditWithFocus(QtWidgets.QTextEdit):
     focusLost = QtCore.Signal()
 
@@ -943,7 +998,7 @@ class Preferences(AbstractDockWidget):
             "such as *.beta packages, with every other package still \n"
             "qualifying for that filter."
         )),
-        qargparse.Integer("clearCacheTimeout", minimum=1, default=10, help=(
+        qargparse.Integer("clearCacheTimeout", min=1, default=10, help=(
             "Clear package repository cache at this interval, in seconds. \n\n"
 
             "Default 10. (Requires restart)\n\n"
@@ -1324,3 +1379,270 @@ class CssHighlighter(QtGui.QSyntaxHighlighter):
             )
             startIndex = self.comment_start_exp.indexIn(
                 text, startIndex + commentLength)
+
+
+class ProfileView(QtWidgets.QTreeView):
+
+    activated = QtCore.Signal(str)
+
+    def __init__(self, parent=None):
+        super(ProfileView, self).__init__(parent)
+
+        self.setHeaderHidden(True)
+        self.setSortingEnabled(True)
+        self.sortByColumn(0, QtCore.Qt.AscendingOrder)
+        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+
+        self.customContextMenuRequested.connect(self.on_context_menu)
+
+    def on_context_menu(self, position):
+        index = self.indexAt(position)
+
+        if not index.isValid():
+            # Clicked outside any item
+            return
+
+        model_ = index.model()
+        menu = QtWidgets.QMenu(self)
+
+        name = str(model_.data(index, model.NameRole))
+
+        activate = QtWidgets.QAction("Activate", menu)
+        activate.triggered.connect(lambda: self.on_activate(name))
+        menu.addAction(activate)
+
+        menu.move(QtGui.QCursor.pos())
+        menu.show()
+
+    def on_activate(self, profile):
+        self.activated.emit(profile)
+
+    def selected_index(self):
+        return self.selectionModel().currentIndex()
+
+    def selected_profile(self):
+        index = self.selected_index()
+        if index.isValid():
+            return index.data(model.NameRole)
+
+
+class Profiles(AbstractDockWidget):
+    """Listing and changing profiles"""
+
+    icon = "Default_Profile"
+
+    profile_changed = QtCore.Signal(str)
+    version_changed = QtCore.Signal(str)
+    reset = QtCore.Signal()
+
+    def __init__(self, ctrl, parent=None):
+        super(Profiles, self).__init__("Profiles", parent)
+        self.setAttribute(QtCore.Qt.WA_StyledBackground)
+        self.setObjectName("Profiles")
+
+        panels = {
+            "central": QtWidgets.QWidget(),
+            "head": QtWidgets.QWidget(),
+            "body": QtWidgets.QWidget(),
+        }
+
+        widgets = {
+            # head: current profile
+            "icon": QtWidgets.QLabel(),
+            "name": QtWidgets.QLabel(),
+            "version": LineEditWithCompleter(),
+
+            # body: profiles view and toolset
+            "refresh": QtWidgets.QPushButton(""),
+            "search": QtWidgets.QLineEdit(),
+            "view": ProfileView(),
+
+            "tools": QtWidgets.QWidget(),
+            "favorite": QtWidgets.QPushButton(""),
+            "filtering": QtWidgets.QPushButton(""),
+            "expand": QtWidgets.QPushButton(""),
+            "collapse": QtWidgets.QPushButton(""),
+            "sep1": QtWidgets.QFrame(),
+            "versioning": QtWidgets.QPushButton(""),
+        }
+
+        models = {
+            "source": None,
+            "proxy": model.ProfileProxyModel(),
+        }
+
+        layout = QtWidgets.QGridLayout(panels["head"])
+        layout.setContentsMargins(0, 0, 0, 0)
+        layout.addWidget(widgets["icon"], 0, 0, 2, 1)
+        layout.addWidget(widgets["name"], 0, 1)
+        layout.addWidget(widgets["version"], 1, 1)
+
+        layout = QtWidgets.QVBoxLayout(widgets["tools"])
+        layout.setContentsMargins(0, 2, 0, 0)
+        layout.addWidget(widgets["favorite"])
+        layout.addWidget(widgets["filtering"])
+        layout.addWidget(widgets["expand"])
+        layout.addWidget(widgets["collapse"])
+        layout.addWidget(widgets["sep1"])
+        layout.addWidget(widgets["versioning"])
+        # (epic) quick make profile
+        # (epic) quick edit profile
+        # (epic) remove or hide local profile
+        layout.addStretch()
+
+        layout = QtWidgets.QGridLayout(panels["body"])
+        layout.setContentsMargins(0, 0, 0, 0)
+        layout.addWidget(widgets["refresh"], 0, 0)
+        layout.addWidget(widgets["tools"], 1, 0)
+        layout.addWidget(widgets["search"], 0, 1)
+        layout.addWidget(widgets["view"], 1, 1)
+
+        layout = QtWidgets.QVBoxLayout(panels["central"])
+        layout.setContentsMargins(4, 4, 4, 4)
+        layout.addWidget(panels["head"])
+        layout.addWidget(panels["body"], stretch=True)
+
+        version = widgets["version"]
+        search = widgets["search"]
+        view = widgets["view"]
+        proxy = models["proxy"]
+
+        view.setModel(proxy)
+        selection = view.selectionModel()
+
+        proxy.setFilterCaseSensitivity(QtCore.Qt.CaseInsensitive)
+        search.setPlaceholderText("Filter profiles..")
+
+        version.setToolTip("Click to change profile version")
+        version.setEnabled(False)
+
+        widgets["sep1"].setFrameShape(QtWidgets.QFrame.HLine)
+        widgets["sep1"].setFrameShadow(QtWidgets.QFrame.Sunken)
+
+        # icons
+        icon_size = QtCore.QSize(14, 14)
+
+        icon = res.icon("refresh")
+        widgets["refresh"].setIcon(icon)
+        widgets["refresh"].setIconSize(icon_size)
+
+        icon = res.icon("star_bright")
+        icon.addPixmap(res.pixmap("star_dim"), icon.Disabled)
+        widgets["favorite"].setIcon(icon)
+        widgets["favorite"].setIconSize(icon_size)
+
+        icon = QtGui.QIcon()
+        icon.addPixmap(res.pixmap("filter_on"), icon.Normal, icon.On)
+        icon.addPixmap(res.pixmap("filter_off"), icon.Normal, icon.Off)
+        widgets["filtering"].setIcon(icon)
+        widgets["filtering"].setIconSize(icon_size)
+        widgets["filtering"].setCheckable(True)
+        widgets["filtering"].setAutoRepeat(True)
+
+        icon = QtGui.QIcon()
+        icon.addPixmap(res.pixmap("version_on"), icon.Normal, icon.On)
+        icon.addPixmap(res.pixmap("version_off"), icon.Normal, icon.Off)
+        widgets["versioning"].setIcon(icon)
+        widgets["versioning"].setIconSize(icon_size)
+        widgets["versioning"].setCheckable(True)
+        widgets["versioning"].setAutoRepeat(True)
+
+        icon = res.icon("expand")
+        widgets["expand"].setIcon(icon)
+        widgets["expand"].setIconSize(icon_size)
+
+        icon = res.icon("collapse")
+        widgets["collapse"].setIcon(icon)
+        widgets["collapse"].setIconSize(icon_size)
+
+        icon = res.icon("Default_Profile")
+        icon = icon.pixmap(QtCore.QSize(px(32), px(32)))
+        widgets["icon"].setPixmap(icon)
+
+        # signals
+        view.activated.connect(self.profile_changed.emit)
+        selection.currentChanged.connect(self.on_selected_profile_changed)
+        version.changed.connect(self.version_changed.emit)
+        search.textChanged.connect(view.expandAll)
+        search.textChanged.connect(proxy.setFilterFixedString)
+        widgets["refresh"].clicked.connect(self.reset.emit)
+        widgets["favorite"].clicked.connect(self.on_favorite)
+        widgets["filtering"].clicked.connect(self.on_filtering)
+        widgets["versioning"].clicked.connect(
+            lambda *args: version.setEnabled(not version.isEnabled()))
+        widgets["expand"].clicked.connect(view.expandAll)
+        widgets["collapse"].clicked.connect(view.collapseAll)
+        ctrl.resetted.connect(view.expandAll)
+
+        self._widgets = widgets
+        self._models = models
+        self._ctrl = ctrl
+
+        self.setWidget(panels["central"])
+
+        self.update_favorite_btn(None)  # nothing selected on startup
+
+    def set_model(self, profile_model, version_model):
+        # profile
+        proxy = self._models["proxy"]
+        proxy.setSourceModel(profile_model)
+        self._models["source"] = profile_model
+        self._widgets["filtering"].setChecked(profile_model.is_filtering)
+        # version
+        self._widgets["version"].setModel(version_model)
+
+    def on_context_menu(self, window):
+        def _on_context_menu(*args):
+            name = self._models["source"].current
+
+            menu = MenuWithTooltip(window)
+            separator = QtWidgets.QWidgetAction(menu)
+            separator.setDefaultWidget(QtWidgets.QLabel(name))
+            menu.addAction(separator)
+
+            def on_reset():
+                window.reset()
+
+            reset = QtWidgets.QAction("Reset", menu)
+            reset.triggered.connect(on_reset)
+            reset.setToolTip("Re-scan repository for new Rez packages")
+            menu.addAction(reset)
+
+            menu.addSeparator()
+
+            menu.move(QtGui.QCursor.pos())
+            menu.show()
+
+        return _on_context_menu
+
+    def on_favorite(self):
+        model_ = self._models["source"]
+        proxy = self._models["proxy"]
+        view = self._widgets["view"]
+        index = proxy.mapToSource(view.selected_index())
+
+        model_.update_favorite(self._ctrl, index)
+        model_.update_profile_icon(index)
+
+    def on_filtering(self):
+        proxy = self._models["proxy"]
+        model_ = self._models["source"]
+        model_.is_filtering = not model_.is_filtering
+
+        proxy.invalidateFilter()
+
+    def on_selected_profile_changed(self):
+        view = self._widgets["view"]
+        self.update_favorite_btn(view.selected_profile())
+
+    def update_favorite_btn(self, profile):
+        btn = self._widgets["favorite"]
+        btn.setEnabled(bool(profile))
+
+    def update_current(self, profile, version, icon):
+        self._widgets["name"].setText(profile)
+        self._widgets["version"].setText(version)
+
+        icon = res.icon(icon)
+        icon = icon.pixmap(QtCore.QSize(px(32), px(32)))
+        self._widgets["icon"].setPixmap(icon)
